@@ -3,9 +3,10 @@
 import os
 import sys
 import shlex
+import inspect
 import argparse
 import builtins
-import collections.abc as abc
+import collections.abc as cabc
 
 from xonsh.lazyasd import lazyobject
 from xonsh.dirstack import cd, pushd, popd, dirs, _get_cwd
@@ -14,7 +15,7 @@ from xonsh.foreign_shells import foreign_shell_data
 from xonsh.jobs import jobs, fg, bg, clean_jobs
 from xonsh.history import history_main
 from xonsh.platform import (ON_ANACONDA, ON_DARWIN, ON_WINDOWS, ON_FREEBSD,
-    scandir)
+                            ON_NETBSD, scandir)
 from xonsh.proc import foreground
 from xonsh.replay import replay_main
 from xonsh.timings import timeit_alias
@@ -22,10 +23,11 @@ from xonsh.tools import (XonshError, argvquote, escape_windows_cmd_string,
                          to_bool)
 from xonsh.xontribs import xontribs_main
 from xonsh.xoreutils import _which
-from xonsh.completers._aliases import completer_alias
+
+import xonsh.completers._aliases as xca
 
 
-class Aliases(abc.MutableMapping):
+class Aliases(cabc.MutableMapping):
     """Represents a location to hold and look up aliases."""
 
     def __init__(self, *args, **kwargs):
@@ -43,7 +45,7 @@ class Aliases(abc.MutableMapping):
         val = self._raw.get(key)
         if val is None:
             return default
-        elif isinstance(val, abc.Iterable) or callable(val):
+        elif isinstance(val, cabc.Iterable) or callable(val):
             return self.eval_alias(val, seen_tokens={key})
         else:
             msg = 'alias of {!r} has an inappropriate type: {!r}'
@@ -92,7 +94,7 @@ class Aliases(abc.MutableMapping):
         """
         word = line.split(' ', 1)[0]
         if word in builtins.aliases and isinstance(self.get(word),
-                                                   abc.Sequence):
+                                                   cabc.Sequence):
             word_idx = line.find(word)
             expansion = ' '.join(self.get(word))
             line = line[:word_idx] + expansion + line[word_idx+len(word):]
@@ -142,7 +144,6 @@ class Aliases(abc.MutableMapping):
                 p.pretty(dict(self))
 
 
-
 def xonsh_exit(args, stdin=None):
     """Sends signal to exit shell."""
     if not clean_jobs():
@@ -250,11 +251,15 @@ def source_foreign(args, stdin=None):
 def source_alias(args, stdin=None):
     """Executes the contents of the provided files in the current context.
     If sourced file isn't found in cwd, search for file along $PATH to source
-    instead"""
+    instead.
+    """
+    env = builtins.__xonsh_env__
+    encoding = env.get('XONSH_ENCODING')
+    errors = env.get('XONSH_ENCODING_ERRORS')
     for fname in args:
         if not os.path.isfile(fname):
             fname = locate_binary(fname)
-        with open(fname, 'r') as fp:
+        with open(fname, 'r', encoding=encoding, errors=errors) as fp:
             builtins.execx(fp.read(), 'exec', builtins.__xonsh_ctx__)
 
 
@@ -280,26 +285,45 @@ def source_cmd(args, stdin=None):
 
 
 def xexec(args, stdin=None):
-    """Replaces current process with command specified and passes in the
-    current xonsh environment.
+    """exec [-h|--help] command [args...]
+
+    exec (also aliased as xexec) uses the os.execvpe() function to
+    replace the xonsh process with the specified program. This provides
+    the functionality of the bash 'exec' builtin::
+
+        >>> exec bash -l -i
+        bash $
+
+    The '-h' and '--help' options print this message and exit.
+
+    Notes
+    -----
+    This command **is not** the same as the Python builtin function
+    exec(). That function is for running Python code. This command,
+    which shares the same name as the sh-lang statement, is for launching
+    a command directly in the same process. In the event of a name conflict,
+    please use the xexec command directly or dive into subprocess mode
+    explicitly with ![exec command]. For more details, please see
+    http://xon.sh/faq.html#exec.
     """
-    env = builtins.__xonsh_env__
-    denv = env.detype()
-    if len(args) > 0:
+    if len(args) == 0:
+        return (None, 'xonsh: exec: no args specified\n', 1)
+    elif args[0] == '-h' or args[0] == '--help':
+        return inspect.getdoc(xexec)
+    else:
+        denv = builtins.__xonsh_env__.detype()
         try:
             os.execvpe(args[0], args, denv)
         except FileNotFoundError as e:
             return (None, 'xonsh: exec: file not found: {}: {}'
                           '\n'.format(e.args[1], args[0]), 1)
-    else:
-        return (None, 'xonsh: exec: no args specified\n', 1)
 
 
 @lazyobject
 def _BANG_N_PARSER():
-    parser = argparse.ArgumentParser('!n', usage='!n <n>',
-                description="Re-runs the nth command as specified in the "
-                            "argument.")
+    parser = argparse.ArgumentParser(
+        '!n', usage='!n <n>',
+        description="Re-runs the nth command as specified in the argument.")
     parser.add_argument('n', type=int, help='the command to rerun, may be '
                                             'negative')
     return parser
@@ -326,6 +350,7 @@ def bang_bang(args, stdin=None):
 
 class AWitchAWitch(argparse.Action):
     SUPPRESS = '==SUPPRESS=='
+
     def __init__(self, option_strings, version=None, dest=SUPPRESS,
                  default=SUPPRESS, **kwargs):
         super().__init__(option_strings=option_strings, dest=dest,
@@ -348,7 +373,7 @@ def which(args, stdin=None, stdout=None, stderr=None):
     parser = argparse.ArgumentParser('which', description=desc)
     parser.add_argument('args', type=str, nargs='+',
                         help='The executables or aliases to search for')
-    parser.add_argument('-a','--all', action='store_true', dest='all',
+    parser.add_argument('-a', '--all', action='store_true', dest='all',
                         help='Show all matches in $PATH and xonsh.aliases')
     parser.add_argument('-s', '--skip-alias', action='store_true',
                         help='Do not search in xonsh.aliases', dest='skip')
@@ -397,12 +422,14 @@ def which(args, stdin=None, stdout=None, stderr=None):
         # skip alias check if user asks to skip
         if (arg in builtins.aliases and not pargs.skip):
             if pargs.plain or not pargs.verbose:
-                if isinstance(builtins.aliases[arg], list):
+                if not callable(builtins.aliases[arg]):
                     print(' '.join(builtins.aliases[arg]), file=stdout)
                 else:
                     print(arg, file=stdout)
             else:
                 print("aliases['{}'] = {}".format(arg, builtins.aliases[arg]), file=stdout)
+                if callable(builtins.aliases[arg]):
+                    builtins.__xonsh_superhelp__(builtins.aliases[arg])
             nmatches += 1
             if not pargs.all:
                 continue
@@ -490,6 +517,7 @@ def make_default_aliases():
         'EOF': xonsh_exit,
         'exit': xonsh_exit,
         'quit': xonsh_exit,
+        'exec': xexec,
         'xexec': xexec,
         'source': source_alias,
         'source-zsh': ['source-foreign', 'zsh', '--sourcer=source'],
@@ -508,7 +536,7 @@ def make_default_aliases():
         'ipynb': ['jupyter', 'notebook', '--no-browser'],
         'which': which,
         'xontrib': xontribs_main,
-        'completer': completer_alias
+        'completer': xca.completer_alias
     }
     if ON_WINDOWS:
         # Borrow builtin commands from cmd.exe.
@@ -565,6 +593,10 @@ def make_default_aliases():
         default_aliases['egrep'] = ['egrep', '--color=auto']
         default_aliases['fgrep'] = ['fgrep', '--color=auto']
         default_aliases['ls'] = ['ls', '-G']
+    elif ON_NETBSD:
+        default_aliases['grep'] = ['grep', '--color=auto']
+        default_aliases['egrep'] = ['egrep', '--color=auto']
+        default_aliases['fgrep'] = ['fgrep', '--color=auto']
     else:
         default_aliases['grep'] = ['grep', '--color=auto']
         default_aliases['egrep'] = ['egrep', '--color=auto']
