@@ -2,17 +2,19 @@
 """The xonsh abstract syntax tree node."""
 # These are imported into our module namespace for the benefit of parser.py.
 # pylint: disable=unused-import
-from ast import Module, Num, Expr, Str, Bytes, UnaryOp, UAdd, USub, Invert, \
-    BinOp, Add, Sub, Mult, Div, FloorDiv, Mod, Pow, Compare, Lt, Gt, \
-    LtE, GtE, Eq, NotEq, In, NotIn, Is, IsNot, Not, BoolOp, Or, And, \
-    Subscript, Load, Slice, ExtSlice, List, Tuple, Set, Dict, AST, NameConstant, \
-    Name, GeneratorExp, Store, comprehension, ListComp, SetComp, DictComp, \
-    Assign, AugAssign, BitXor, BitAnd, BitOr, LShift, RShift, Assert, Delete, \
-    Del, Pass, Raise, Import, alias, ImportFrom, Continue, Break, Yield, \
-    YieldFrom, Return, IfExp, Lambda, arguments, arg, Call, keyword, \
-    Attribute, Global, Nonlocal, If, While, For, withitem, With, Try, \
-    ExceptHandler, FunctionDef, ClassDef, Starred, NodeTransformer, \
+from ast import (
+    Module, Num, Expr, Str, Bytes, UnaryOp, UAdd, USub, Invert,
+    BinOp, Add, Sub, Mult, Div, FloorDiv, Mod, Pow, Compare, Lt, Gt,
+    LtE, GtE, Eq, NotEq, In, NotIn, Is, IsNot, Not, BoolOp, Or, And,
+    Subscript, Load, Slice, ExtSlice, List, Tuple, Set, Dict, AST, NameConstant,
+    Name, GeneratorExp, Store, comprehension, ListComp, SetComp, DictComp,
+    Assign, AugAssign, BitXor, BitAnd, BitOr, LShift, RShift, Assert, Delete,
+    Del, Pass, Raise, Import, alias, ImportFrom, Continue, Break, Yield,
+    YieldFrom, Return, IfExp, Lambda, arguments, arg, Call, keyword,
+    Attribute, Global, Nonlocal, If, While, For, withitem, With, Try,
+    ExceptHandler, FunctionDef, ClassDef, Starred, NodeTransformer,
     Interactive, Expression, Index, literal_eval, dump, walk, increment_lineno
+)
 from ast import Ellipsis as EllipsisNode
 # pylint: enable=unused-import
 import textwrap
@@ -224,12 +226,13 @@ class CtxAwareTransformer(NodeTransformer):
 
     def is_in_scope(self, node):
         """Determines whether or not the current node is in scope."""
-        lname = leftmostname(node)
-        if lname is None:
-            return node
+        names = gather_names(node)
+        if not names:
+            return True
         inscope = False
         for ctx in reversed(self.contexts):
-            if lname in ctx:
+            names -= ctx
+            if not names:
                 inscope = True
                 break
         return inscope
@@ -429,6 +432,13 @@ class CtxAwareTransformer(NodeTransformer):
         """Handle visiting a function definition."""
         self.ctxadd(node.name)
         self.contexts.append(set())
+        args = node.args
+        argchain = [args.args, args.kwonlyargs]
+        if args.vararg is not None:
+            argchain.append((args.vararg,))
+        if args.kwarg is not None:
+            argchain.append((args.kwarg,))
+        self.ctxupdate(a.arg for a in itertools.chain.from_iterable(argchain))
         self.generic_visit(node)
         self.contexts.pop()
         return node
@@ -464,7 +474,6 @@ class CtxAwareTransformer(NodeTransformer):
         return node
 
 
-
 def pdump(s, **kwargs):
     """performs a pretty dump of an AST node."""
     if isinstance(s, AST):
@@ -474,7 +483,7 @@ def pdump(s, **kwargs):
     lens = len(s) + 1
     if lens == 1:
         return s
-    i = min([s.find(o)%lens for o in openers])
+    i = min([s.find(o) % lens for o in openers])
     if i == lens - 1:
         return s
     closer = closers[openers.find(s[i])]
@@ -490,7 +499,7 @@ def pdump(s, **kwargs):
     return pre + mid + post
 
 
-def pprint(s, *, sep=None, end=None, file=None, flush=False, **kwargs):
+def pprint_ast(s, *, sep=None, end=None, file=None, flush=False, **kwargs):
     """Performs a pretty print of the AST nodes."""
     print(pdump(s, **kwargs), sep=sep, end=end, file=file, flush=flush)
 
