@@ -7,13 +7,13 @@ import time
 import builtins
 
 from xonsh.tools import (XonshError, escape_windows_cmd_string, print_exception,
-    DefaultNotGiven)
+                         DefaultNotGiven, check_for_partial_string)
 from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS
 from xonsh.codecache import (should_use_cache, code_cache_name,
                              code_cache_check, get_cache_filename,
                              update_cache, run_compiled_code)
 from xonsh.completer import Completer
-from xonsh.environ import multiline_prompt, format_prompt, partial_format_prompt
+from xonsh.environ import multiline_prompt, partial_format_prompt
 
 
 class _TeeOut(object):
@@ -23,6 +23,7 @@ class _TeeOut(object):
         self.buffer = buf
         self.stdout = sys.stdout
         self.encoding = self.stdout.encoding
+        self.errors = self.stdout.errors
         sys.stdout = self
 
     def __del__(self):
@@ -34,7 +35,7 @@ class _TeeOut(object):
 
     def write(self, data):
         """Writes data to the original stdout and the buffer."""
-        #data = data.replace('\001', '').replace('\002', '')
+        # data = data.replace('\001', '').replace('\002', '')
         self.stdout.write(data)
         self.buffer.write(data)
 
@@ -55,6 +56,7 @@ class _TeeErr(object):
         self.buffer = buf
         self.stderr = sys.stderr
         self.encoding = self.stderr.encoding
+        self.errors = self.stderr.errors
         sys.stderr = self
 
     def __del__(self):
@@ -66,7 +68,7 @@ class _TeeErr(object):
 
     def write(self, data):
         """Writes data to the original stderr and the buffer."""
-        #data = data.replace('\001', '').replace('\002', '')
+        # data = data.replace('\001', '').replace('\002', '')
         self.stderr.write(data)
         self.buffer.write(data)
 
@@ -178,6 +180,7 @@ class BaseShell(object):
             ts1 = ts1 or time.time()
             self._append_history(inp=src, ts=[ts0, ts1], tee_out=tee.getvalue())
             tee.close()
+            builtins.__xonsh_env__['PWD'] = os.getcwd()        # track any change in working directory
         if builtins.__xonsh_exit__:  # pylint: disable=no-member
             return True
 
@@ -207,7 +210,10 @@ class BaseShell(object):
                 update_cache(code, cachefname)
             self.reset_buffer()
         except SyntaxError:
-            if line == '\n':
+            partial_string_info = check_for_partial_string(src)
+            in_partial_string = (partial_string_info[0] is not None and
+                                 partial_string_info[1] is None)
+            if ((line == '\n' and not in_partial_string)):
                 self.reset_buffer()
                 print_exception()
                 return src, None
@@ -226,22 +232,26 @@ class BaseShell(object):
 
     def settitle(self):
         """Sets terminal title."""
-        _ = self
         env = builtins.__xonsh_env__  # pylint: disable=no-member
         term = env.get('TERM', None)
         # Shells running in emacs sets TERM to "dumb" or "eterm-color".
         # Do not set title for these to avoid garbled prompt.
-        if term is None or term in ['dumb', 'eterm-color', 'linux']:
+        if (term is None and not ON_WINDOWS) or term in ['dumb', 'eterm-color',
+                                                         'linux']:
             return
         t = env.get('TITLE')
         if t is None:
             return
-        t = format_prompt(t)
+        t = partial_format_prompt(t)
         if ON_WINDOWS and 'ANSICON' not in env:
             t = escape_windows_cmd_string(t)
             os.system('title {}'.format(t))
         else:
-            os.write(1, "\x1b]2;{0}\x07".format(t).encode())
+            with open(1, 'wb', closefd=False) as f:
+                # prevent xonsh from answering interative questions
+                # on the next command by writing the title
+                f.write("\x1b]0;{0}\x07".format(t).encode())
+                f.flush()
 
     @property
     def prompt(self):
@@ -265,7 +275,6 @@ class BaseShell(object):
 
     def _append_history(self, tee_out=None, **info):
         """Append information about the command to the history."""
-        _ = self
         hist = builtins.__xonsh_history__  # pylint: disable=no-member
         info['rtn'] = hist.last_cmd_rtn
         tee_out = tee_out or None
