@@ -1,6 +1,5 @@
 """Amalgamation of xonsh.completers package, made up of the following modules, in order:
 
-* bash
 * completer
 * pip
 * tools
@@ -10,6 +9,7 @@
 * path
 * python
 * base
+* bash
 * dirs
 * init
 
@@ -58,201 +58,10 @@ class _LazyModule(_ModuleType):
             dct['loaded'] = True
         return getattr(m, name)
 
-#
-# bash
-#
-os = _LazyModule.load('os', 'os')
-re = _LazyModule.load('re', 're')
-shlex = _LazyModule.load('shlex', 'shlex')
-pickle = _LazyModule.load('pickle', 'pickle')
-hashlib = _LazyModule.load('hashlib', 'hashlib')
-pathlib = _LazyModule.load('pathlib', 'pathlib')
-builtins = _LazyModule.load('builtins', 'builtins')
-subprocess = _LazyModule.load('subprocess', 'subprocess')
-xl = _LazyModule.load('xonsh', 'xonsh.lazyasd', 'xl')
-xp = _LazyModule.load('xonsh', 'xonsh.platform', 'xp')
-RE_DASHF = xl.LazyObject(lambda: re.compile(r'-F\s+(\w+)'),
-                         globals(), 'RE_DASHF')
-
-INITED = False
-
-BASH_COMPLETE_HASH = None
-BASH_COMPLETE_FUNCS = {}
-BASH_COMPLETE_FILES = {}
-
-CACHED_HASH = None
-CACHED_FUNCS = None
-CACHED_FILES = None
-
-BASH_COMPLETE_SCRIPT = """source "{filename}"
-COMP_WORDS=({line})
-COMP_LINE={comp_line}
-COMP_POINT=${{#COMP_LINE}}
-COMP_COUNT={end}
-COMP_CWORD={n}
-{func} {cmd} {prefix} {prev}
-for ((i=0;i<${{#COMPREPLY[*]}};i++)) do echo ${{COMPREPLY[i]}}; done
-"""
-
-
-def update_bash_completion():
-    global BASH_COMPLETE_FUNCS, BASH_COMPLETE_FILES, BASH_COMPLETE_HASH
-    global CACHED_FUNCS, CACHED_FILES, CACHED_HASH, INITED
-
-    completers = builtins.__xonsh_env__.get('BASH_COMPLETIONS', ())
-    BASH_COMPLETE_HASH = hashlib.md5(repr(completers).encode()).hexdigest()
-
-    datadir = builtins.__xonsh_env__['XONSH_DATA_DIR']
-    cachefname = os.path.join(datadir, 'bash_completion_cache')
-
-    if not INITED:
-        if os.path.isfile(cachefname):
-            # load from cache
-            with open(cachefname, 'rb') as cache:
-                CACHED_HASH, CACHED_FUNCS, CACHED_FILES = pickle.load(cache)
-                BASH_COMPLETE_HASH = CACHED_HASH
-                BASH_COMPLETE_FUNCS = CACHED_FUNCS
-                BASH_COMPLETE_FILES = CACHED_FILES
-        else:
-            # create initial cache
-            _load_bash_complete_funcs()
-            _load_bash_complete_files()
-            CACHED_HASH = BASH_COMPLETE_HASH
-            CACHED_FUNCS = BASH_COMPLETE_FUNCS
-            CACHED_FILES = BASH_COMPLETE_FILES
-            with open(cachefname, 'wb') as cache:
-                val = (CACHED_HASH, CACHED_FUNCS, CACHED_FILES)
-                pickle.dump(val, cache)
-        INITED = True
-
-    invalid = ((not os.path.isfile(cachefname)) or
-               BASH_COMPLETE_HASH != CACHED_HASH or
-               _completions_time() > os.stat(cachefname).st_mtime)
-
-    if invalid:
-        # update the cache
-        _load_bash_complete_funcs()
-        _load_bash_complete_files()
-        CACHED_HASH = BASH_COMPLETE_HASH
-        CACHED_FUNCS = BASH_COMPLETE_FUNCS
-        CACHED_FILES = BASH_COMPLETE_FILES
-        with open(cachefname, 'wb') as cache:
-            val = (CACHED_HASH, BASH_COMPLETE_FUNCS, BASH_COMPLETE_FILES)
-            pickle.dump(val, cache)
-
-
-def complete_from_bash(prefix, line, begidx, endidx, ctx):
-    """Completes based on results from BASH completion."""
-    update_bash_completion()
-    splt = line.split()
-    cmd = splt[0]
-    func = BASH_COMPLETE_FUNCS.get(cmd, None)
-    fnme = BASH_COMPLETE_FILES.get(cmd, None)
-    if func is None or fnme is None:
-        return set()
-    idx = n = 0
-    for n, tok in enumerate(splt):
-        if tok == prefix:
-            idx = line.find(prefix, idx)
-            if idx >= begidx:
-                break
-        prev = tok
-    if len(prefix) == 0:
-        prefix = '""'
-        n += 1
-    else:
-        prefix = shlex.quote(prefix)
-
-    script = BASH_COMPLETE_SCRIPT.format(
-        filename=fnme, line=' '.join(shlex.quote(p) for p in splt),
-        comp_line=shlex.quote(line), n=n, func=func, cmd=cmd,
-        end=endidx + 1, prefix=prefix, prev=shlex.quote(prev))
-    try:
-        out = subprocess.check_output(
-            [xp.bash_command()], input=script, universal_newlines=True,
-            stderr=subprocess.PIPE, env=builtins.__xonsh_env__.detype())
-    except (subprocess.CalledProcessError, FileNotFoundError):
-        out = ''
-
-    rtn = set(out.splitlines())
-    return rtn
-
-
-def _load_bash_complete_funcs():
-    global BASH_COMPLETE_FUNCS
-    BASH_COMPLETE_FUNCS = bcf = {}
-    inp = _collect_completions_sources()
-    if not inp:
-        return
-    inp.append('complete -p\n')
-    out = _source_completions(inp)
-    for line in out.splitlines():
-        head, _, cmd = line.rpartition(' ')
-        if len(cmd) == 0 or cmd == 'cd':
-            continue
-        m = RE_DASHF.search(head)
-        if m is None:
-            continue
-        bcf[cmd] = m.group(1)
-
-
-def _load_bash_complete_files():
-    global BASH_COMPLETE_FILES
-    inp = _collect_completions_sources()
-    if not inp:
-        BASH_COMPLETE_FILES = {}
-        return
-    if BASH_COMPLETE_FUNCS:
-        inp.append('shopt -s extdebug')
-        bash_funcs = set(BASH_COMPLETE_FUNCS.values())
-        inp.append('declare -F ' + ' '.join([f for f in bash_funcs]))
-        inp.append('shopt -u extdebug\n')
-    out = _source_completions(inp)
-    func_files = {}
-    for line in out.splitlines():
-        parts = line.split()
-        if xp.ON_WINDOWS:
-            parts = [parts[0], ' '.join(parts[2:])]
-        func_files[parts[0]] = parts[-1]
-    BASH_COMPLETE_FILES = {
-        cmd: func_files[func]
-        for cmd, func in BASH_COMPLETE_FUNCS.items()
-        if func in func_files
-    }
-
-
-def _source_completions(source):
-    try:
-        return subprocess.check_output(
-            [xp.bash_command()], input='\n'.join(source),
-            universal_newlines=True, env=builtins.__xonsh_env__.detype(),
-            stderr=subprocess.DEVNULL)
-    except FileNotFoundError:
-        return ''
-
-
-def _collect_completions_sources():
-    sources = []
-    completers = builtins.__xonsh_env__.get('BASH_COMPLETIONS', ())
-    paths = (pathlib.Path(x) for x in completers)
-    for path in paths:
-        if path.is_file():
-            sources.append('source "{}"'.format(path.as_posix()))
-        elif path.is_dir():
-            for _file in (x for x in path.glob('*') if x.is_file()):
-                sources.append('source "{}"'.format(_file.as_posix()))
-    return sources
-
-
-def _completions_time():
-    compfiles = builtins.__xonsh_env__.get('BASH_COMPLETIONS', ())
-    compfiles = [os.stat(x).st_mtime for x in compfiles if os.path.exists(x)]
-    return max(compfiles) if compfiles else 0
-
 #
 # completer
 #
-# amalgamated builtins
+builtins = _LazyModule.load('builtins', 'builtins')
 def complete_completer(prefix, line, start, end, ctx):
     """
     Completion for "completer"
@@ -289,9 +98,9 @@ def complete_completer(prefix, line, start, end, ctx):
 #
 # pip
 #
-# amalgamated re
-# amalgamated subprocess
-# amalgamated xonsh.lazyasd
+re = _LazyModule.load('re', 're')
+subprocess = _LazyModule.load('subprocess', 'subprocess')
+xl = _LazyModule.load('xonsh', 'xonsh.lazyasd', 'xl')
 PIP_RE = xl.LazyObject(lambda: re.compile("pip(?:\d|\.)*"),
                        globals(), 'PIP_RE')
 PIP_LIST_RE = xl.LazyObject(lambda: re.compile("pip(?:\d|\.)* (?:uninstall|show)"),
@@ -546,10 +355,10 @@ POS (optional) is a position into the list of completers at which the new
 #
 # commands
 #
-# amalgamated os
+os = _LazyModule.load('os', 'os')
 # amalgamated builtins
 xt = _LazyModule.load('xonsh', 'xonsh.tools', 'xt')
-# amalgamated xonsh.platform
+xp = _LazyModule.load('xonsh', 'xonsh.platform', 'xp')
 # amalgamated xonsh.completers.tools
 SKIP_TOKENS = {'sudo', 'time', 'timeit', 'which', 'showcmd', 'man'}
 
@@ -604,7 +413,7 @@ def complete_skipper(cmd, line, start, end, ctx):
 #
 # amalgamated os
 # amalgamated re
-# amalgamated pickle
+pickle = _LazyModule.load('pickle', 'pickle')
 # amalgamated builtins
 # amalgamated subprocess
 # amalgamated xonsh.lazyasd
@@ -846,12 +655,11 @@ def _splitpath(path):
 
 def _splitpath_helper(path, sofar=()):
     folder, path = os.path.split(path)
-    if path == "":
+    if path:
+        sofar = sofar + (path, )
+    if not folder or folder == xt.get_sep():
         return sofar[::-1]
-    elif folder == "":
-        return (sofar + (path, ))[::-1]
-    else:
-        return _splitpath_helper(folder, sofar + (path, ))
+    return _splitpath_helper(folder, sofar)
 
 
 def subsequence_match(ref, typed, csc):
@@ -964,7 +772,7 @@ sys = _LazyModule.load('sys', 'sys')
 inspect = _LazyModule.load('inspect', 'inspect')
 # amalgamated builtins
 importlib = _LazyModule.load('importlib', 'importlib')
-abc = _LazyModule.load('collections', 'collections.abc', 'abc')
+cabc = _LazyModule.load('collections', 'collections.abc', 'cabc')
 # amalgamated xonsh.tools
 # amalgamated xonsh.lazyasd
 # amalgamated xonsh.completers.tools
@@ -1051,10 +859,13 @@ def attr_complete(prefix, ctx, filter_func):
         except:  # pylint:disable=bare-except
             continue
         a = getattr(val, opt)
-        if callable(a):
-            rpl = opt + '('
-        elif isinstance(a, abc.Iterable):
-            rpl = opt + '['
+        if builtins.__xonsh_env__['COMPLETIONS_BRACKETS']:
+            if callable(a):
+                rpl = opt + '('
+            elif isinstance(a, (cabc.Sequence, cabc.Mapping)):
+                rpl = opt + '['
+            else:
+                rpl = opt
         else:
             rpl = opt
         # note that prefix[:prelen-len(attr)] != prefix[:-len(attr)]
@@ -1114,6 +925,86 @@ def complete_base(prefix, line, start, end, ctx):
                 complete_command(prefix, line, start, end, ctx))
     return set()
 
+#
+# bash
+#
+shlex = _LazyModule.load('shlex', 'shlex')
+pathlib = _LazyModule.load('pathlib', 'pathlib')
+# amalgamated builtins
+# amalgamated subprocess
+# amalgamated xonsh.platform
+# amalgamated xonsh.completers.path
+BASH_COMPLETE_SCRIPT = r"""
+{sources}
+if (complete -p "{cmd}" 2> /dev/null || echo _minimal) | grep --quiet -e "_minimal"
+then
+    declare -f _completion_loader > /dev/null && _completion_loader "{cmd}"
+fi
+_func=$(complete -p {cmd} | grep -o -e '-F \w\+' | cut -d ' ' -f 2)
+COMP_WORDS=({line})
+COMP_LINE={comp_line}
+COMP_POINT=${{#COMP_LINE}}
+COMP_COUNT={end}
+COMP_CWORD={n}
+$_func {cmd} {prefix} {prev}
+for ((i=0;i<${{#COMPREPLY[*]}};i++)) do echo ${{COMPREPLY[i]}}; done
+"""
+
+
+def complete_from_bash(prefix, line, begidx, endidx, ctx):
+    """Completes based on results from BASH completion."""
+    sources = _collect_completions_sources()
+    if not sources:
+        return set()
+
+    if prefix.startswith('$'):  # do not complete env variables
+        return set()
+
+    splt = line.split()
+    cmd = splt[0]
+    idx = n = 0
+    prev = ''
+    for n, tok in enumerate(splt):
+        if tok == prefix:
+            idx = line.find(prefix, idx)
+            if idx >= begidx:
+                break
+        prev = tok
+    if len(prefix) == 0:
+        prefix = '""'
+        n += 1
+    else:
+        prefix = shlex.quote(prefix)
+
+    script = BASH_COMPLETE_SCRIPT.format(
+        sources='\n'.join(sources), line=' '.join(shlex.quote(p) for p in splt),
+        comp_line=shlex.quote(line), n=n, cmd=cmd,
+        end=endidx + 1, prefix=prefix, prev=shlex.quote(prev),
+    )
+
+    try:
+        out = subprocess.check_output(
+            [xp.bash_command()], input=script, universal_newlines=True,
+            stderr=subprocess.PIPE, env=builtins.__xonsh_env__.detype())
+    except (subprocess.CalledProcessError, FileNotFoundError):
+        out = ''
+
+    rtn = _quote_paths(set(out.splitlines()), '', '')
+    return rtn
+
+
+def _collect_completions_sources():
+    sources = []
+    completers = builtins.__xonsh_env__.get('BASH_COMPLETIONS', ())
+    paths = (pathlib.Path(x) for x in completers)
+    for path in paths:
+        if path.is_file():
+            sources.append('source "{}"'.format(path.as_posix()))
+        elif path.is_dir():
+            for _file in (x for x in path.glob('*') if x.is_file()):
+                sources.append('source "{}"'.format(_file.as_posix()))
+    return sources
+
 #
 # dirs
 #
