@@ -1,22 +1,21 @@
 # -*- coding: utf-8 -*-
 """Tools to help interface with foreign shells, such as Bash."""
 import os
-import sys
 import re
 import json
 import shlex
+import sys
 import tempfile
 import builtins
 import subprocess
 import warnings
 import functools
-import collections.abc as abc
+import collections.abc as cabc
 
-from xonsh.lazyasd import LazyObject
+from xonsh.lazyasd import lazyobject
 from xonsh.tools import to_bool, ensure_string
 from xonsh.platform import ON_WINDOWS, ON_CYGWIN
 
-
 COMMAND = """{seterrprevcmd}
 {prevcmd}
 echo __XONSH_ENV_BEG__
@@ -80,8 +79,11 @@ else
 fi
 echo ${namefile}"""
 
+
 # mapping of shell name alises to keys in other lookup dictionaries.
-CANON_SHELL_NAMES = LazyObject(lambda: {
+@lazyobject
+def CANON_SHELL_NAMES():
+    return {
     'bash': 'bash',
     '/bin/bash': 'bash',
     'zsh': 'zsh',
@@ -89,55 +91,79 @@ CANON_SHELL_NAMES = LazyObject(lambda: {
     '/usr/bin/zsh': 'zsh',
     'cmd': 'cmd',
     'cmd.exe': 'cmd',
-    }, globals(), 'CANON_SHELL_NAMES')
+    }
 
-DEFAULT_ENVCMDS = LazyObject(lambda: {
+
+@lazyobject
+def DEFAULT_ENVCMDS():
+    return {
     'bash': 'env',
     'zsh': 'env',
     'cmd': 'set',
-    }, globals(), 'DEFAULT_ENVCMDS')
+    }
 
-DEFAULT_ALIASCMDS = LazyObject(lambda: {
+
+@lazyobject
+def DEFAULT_ALIASCMDS():
+    return {
     'bash': 'alias',
     'zsh': 'alias -L',
     'cmd': '',
-    }, globals(), 'DEFAULT_ALIASCMDS')
+    }
 
-DEFAULT_FUNCSCMDS = LazyObject(lambda: {
+
+@lazyobject
+def DEFAULT_FUNCSCMDS():
+    return {
     'bash': DEFAULT_BASH_FUNCSCMD,
     'zsh': DEFAULT_ZSH_FUNCSCMD,
     'cmd': '',
-    }, globals(), 'DEFAULT_FUNCSCMDS')
+    }
 
-DEFAULT_SOURCERS = LazyObject(lambda: {
+
+@lazyobject
+def DEFAULT_SOURCERS():
+    return {
     'bash': 'source',
     'zsh': 'source',
     'cmd': 'call',
-    }, globals(), 'DEFAULT_SOURCERS')
+    }
 
-DEFAULT_TMPFILE_EXT = LazyObject(lambda: {
+
+@lazyobject
+def DEFAULT_TMPFILE_EXT():
+    return {
     'bash': '.sh',
     'zsh': '.zsh',
     'cmd': '.bat',
-    }, globals(), 'DEFAULT_TMPFILE_EXT')
+    }
 
-DEFAULT_RUNCMD = LazyObject(lambda: {
+
+@lazyobject
+def DEFAULT_RUNCMD():
+    return {
     'bash': '-c',
     'zsh': '-c',
     'cmd': '/C',
-    }, globals(), 'DEFAULT_RUNCMD')
+    }
 
-DEFAULT_SETERRPREVCMD = LazyObject(lambda: {
+
+@lazyobject
+def DEFAULT_SETERRPREVCMD():
+    return {
     'bash': 'set -e',
     'zsh': 'set -e',
     'cmd': '@echo off',
-    }, globals(), 'DEFAULT_SETERRPREVCMD')
+    }
 
-DEFAULT_SETERRPOSTCMD = LazyObject(lambda: {
+
+@lazyobject
+def DEFAULT_SETERRPOSTCMD():
+    return {
     'bash': '',
     'zsh': '',
     'cmd': 'if errorlevel 1 exit 1',
-    }, globals(), 'DEFAULT_SETERRPOSTCMD')
+    }
 
 
 @functools.lru_cache()
@@ -224,9 +250,9 @@ def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
     tmpfile_ext = DEFAULT_TMPFILE_EXT.get(shkey, 'sh') if tmpfile_ext is None else tmpfile_ext
     runcmd = DEFAULT_RUNCMD.get(shkey, '-c') if runcmd is None else runcmd
     seterrprevcmd = DEFAULT_SETERRPREVCMD.get(shkey, '') \
-                        if seterrprevcmd is None else seterrprevcmd
+        if seterrprevcmd is None else seterrprevcmd
     seterrpostcmd = DEFAULT_SETERRPOSTCMD.get(shkey, '') \
-                        if seterrpostcmd is None else seterrpostcmd
+        if seterrpostcmd is None else seterrpostcmd
     command = COMMAND.format(envcmd=envcmd, aliascmd=aliascmd, prevcmd=prevcmd,
                              postcmd=postcmd, funcscmd=funcscmd,
                              seterrprevcmd=seterrprevcmd,
@@ -246,7 +272,7 @@ def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
     try:
         s = subprocess.check_output(cmd, stderr=subprocess.PIPE, env=currenv,
                                     # start new session to avoid hangs
-                                    #(doesn't work on Cygwin though)
+                                    # (doesn't work on Cygwin though)
                                     start_new_session=(not ON_CYGWIN),
                                     universal_newlines=True)
     except (subprocess.CalledProcessError, FileNotFoundError):
@@ -263,12 +289,16 @@ def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
     return env, aliases
 
 
-ENV_RE = LazyObject(lambda: re.compile('__XONSH_ENV_BEG__\n(.*)'
-                                       '__XONSH_ENV_END__', flags=re.DOTALL),
-                    globals(), 'ENV_RE')
-ENV_SPLIT_RE = LazyObject(lambda: re.compile('^([^=]+)=([^=]*|[^\n]*)$',
-                                             flags=re.DOTALL|re.MULTILINE),
-                          globals(), 'ENV_SPLIT_RE')
+@lazyobject
+def ENV_RE():
+    return re.compile('__XONSH_ENV_BEG__\n(.*)'
+                      '__XONSH_ENV_END__', flags=re.DOTALL)
+
+
+@lazyobject
+def ENV_SPLIT_RE():
+    return re.compile('^([^=]+)=([^=]*|[^\n]*)$',
+                      flags=re.DOTALL | re.MULTILINE)
 
 
 def parse_env(s):
@@ -282,10 +312,11 @@ def parse_env(s):
     return env
 
 
-ALIAS_RE = LazyObject(lambda: re.compile('__XONSH_ALIAS_BEG__\n(.*)'
-                                         '__XONSH_ALIAS_END__',
-                                         flags=re.DOTALL),
-                      globals(), 'ALIAS_RE')
+@lazyobject
+def ALIAS_RE():
+    return re.compile('__XONSH_ALIAS_BEG__\n(.*)'
+                      '__XONSH_ALIAS_END__',
+                      flags=re.DOTALL)
 
 
 def parse_aliases(s):
@@ -314,10 +345,11 @@ def parse_aliases(s):
     return aliases
 
 
-FUNCS_RE = LazyObject(lambda: re.compile('__XONSH_FUNCS_BEG__\n(.+)\n'
-                                         '__XONSH_FUNCS_END__',
-                                         flags=re.DOTALL),
-                      globals(), 'FUNCS_RE')
+@lazyobject
+def FUNCS_RE():
+    return re.compile('__XONSH_FUNCS_BEG__\n(.+)\n'
+                      '__XONSH_FUNCS_END__',
+                      flags=re.DOTALL)
 
 
 def parse_funcs(s, shell, sourcer=None):
@@ -342,11 +374,11 @@ def parse_funcs(s, shell, sourcer=None):
         warnings.warn(msg.format(exc, shell, s, g1), RuntimeWarning)
         return {}
     sourcer = DEFAULT_SOURCERS.get(shell, 'source') if sourcer is None \
-                                                    else sourcer
+        else sourcer
     funcs = {}
     for funcname, filename in namefiles.items():
-        if funcname.startswith('_'):
-            continue  # skip private functions
+        if funcname.startswith('_') or not filename:
+            continue  # skip private functions and invalid files
         if not os.path.isabs(filename):
             filename = os.path.abspath(filename)
         wrapper = ForeignShellFunctionAlias(name=funcname, shell=shell,
@@ -377,7 +409,7 @@ class ForeignShellFunctionAlias(object):
             Command to source foreing files with.
         """
         sourcer = DEFAULT_SOURCERS.get(shell, 'source') if sourcer is None \
-                                                        else sourcer
+            else sourcer
         self.name = name
         self.shell = shell
         self.filename = filename
@@ -385,7 +417,7 @@ class ForeignShellFunctionAlias(object):
 
     def __eq__(self, other):
         if not hasattr(other, 'name') or not hasattr(other, 'shell') or \
-           not hasattr(other, 'filename') or not hasattr(other, 'sourcer'):
+                not hasattr(other, 'filename') or not hasattr(other, 'sourcer'):
             return NotImplemented
         return (self.name == other.name) and (self.shell == other.shell) and \
                (self.filename == other.filename) and (self.sourcer == other.sourcer)
@@ -416,15 +448,18 @@ class ForeignShellFunctionAlias(object):
         return args, True
 
 
-VALID_SHELL_PARAMS = LazyObject(lambda: frozenset([
-                                    'shell', 'interactive', 'login', 'envcmd',
-                                    'aliascmd', 'extra_args', 'currenv', 'safe',
-                                    'prevcmd', 'postcmd', 'funcscmd', 'sourcer',
-                                    ]), globals(), 'VALID_SHELL_PARAMS')
+@lazyobject
+def VALID_SHELL_PARAMS():
+    return frozenset([
+    'shell', 'interactive', 'login', 'envcmd',
+    'aliascmd', 'extra_args', 'currenv', 'safe',
+    'prevcmd', 'postcmd', 'funcscmd', 'sourcer',
+    ])
+
 
 def ensure_shell(shell):
     """Ensures that a mapping follows the shell specification."""
-    if not isinstance(shell, abc.MutableMapping):
+    if not isinstance(shell, cabc.MutableMapping):
         shell = dict(shell)
     shell_keys = set(shell.keys())
     if not (shell_keys <= VALID_SHELL_PARAMS):
@@ -437,17 +472,17 @@ def ensure_shell(shell):
         shell['login'] = to_bool(shell['login'])
     if 'envcmd' in shell_keys:
         shell['envcmd'] = None if shell['envcmd'] is None \
-                               else ensure_string(shell['envcmd'])
+            else ensure_string(shell['envcmd'])
     if 'aliascmd' in shell_keys:
         shell['aliascmd'] = None if shell['aliascmd'] is None \
-                                 else ensure_string(shell['aliascmd'])
+            else ensure_string(shell['aliascmd'])
     if 'extra_args' in shell_keys and not isinstance(shell['extra_args'], tuple):
         shell['extra_args'] = tuple(map(ensure_string, shell['extra_args']))
     if 'currenv' in shell_keys and not isinstance(shell['currenv'], tuple):
         ce = shell['currenv']
-        if isinstance(ce, abc.Mapping):
+        if isinstance(ce, cabc.Mapping):
             ce = tuple([(ensure_string(k), v) for k, v in ce.items()])
-        elif isinstance(ce, Sequence):
+        elif isinstance(ce, cabc.Sequence):
             ce = tuple([(ensure_string(k), v) for k, v in ce])
         else:
             raise RuntimeError('unrecognized type for currenv')
@@ -460,16 +495,16 @@ def ensure_shell(shell):
         shell['postcmd'] = ensure_string(shell['postcmd'])
     if 'funcscmd' in shell_keys:
         shell['funcscmd'] = None if shell['funcscmd'] is None \
-                                 else ensure_string(shell['funcscmd'])
+            else ensure_string(shell['funcscmd'])
     if 'sourcer' in shell_keys:
         shell['sourcer'] = None if shell['sourcer'] is None \
-                                 else ensure_string(shell['sourcer'])
+            else ensure_string(shell['sourcer'])
     if 'seterrprevcmd' in shell_keys:
         shell['seterrprevcmd'] = None if shell['seterrprevcmd'] is None \
-                                 else ensure_string(shell['seterrprevcmd'])
+            else ensure_string(shell['seterrprevcmd'])
     if 'seterrpostcmd' in shell_keys:
         shell['seterrpostcmd'] = None if shell['seterrpostcmd'] is None \
-                                 else ensure_string(shell['seterrpostcmd'])
+            else ensure_string(shell['seterrpostcmd'])
     return shell
 
 
@@ -541,9 +576,15 @@ def load_foreign_aliases(shells=None, config=None, issue_warning=True):
     """
     shells = _get_shells(shells=shells, config=config, issue_warning=issue_warning)
     aliases = {}
+    xonsh_aliases = builtins.aliases
     for shell in shells:
         shell = ensure_shell(shell)
         _, shaliases = foreign_shell_data(**shell)
-        if shaliases:
-            aliases.update(shaliases)
+        for alias in set(shaliases) & set(xonsh_aliases):
+            del shaliases[alias]
+            print('aliases: alias {!r} of shell {!r} '
+                  'tries to override xonsh alias, '
+                  'xonsh wins!'.format(alias, shell['shell']),
+                  file=sys.stderr)
+        aliases.update(shaliases)
     return aliases
