@@ -2,23 +2,22 @@
 """Implements the xonsh history object."""
 import os
 import sys
-import uuid
-import time
-import json
 import glob
+import json
+import time
+import uuid
 import argparse
-import operator
-import datetime
 import builtins
+import collections
+import datetime
 import functools
 import threading
-import collections
-import collections.abc as abc
+import collections.abc as cabc
 
-from xonsh.lazyasd import LazyObject, lazyobject
+from xonsh.lazyasd import lazyobject
 from xonsh.lazyjson import LazyJSON, ljdump, LJNode
-from xonsh.tools import (ensure_slice, to_history_tuple,
-                         expanduser_abs_path)
+from xonsh.tools import (ensure_slice, to_history_tuple, is_string,
+                         get_portions, expanduser_abs_path, ensure_timestamp)
 from xonsh.diff_history import _dh_create_parser, _dh_main_action
 
 
@@ -115,7 +114,6 @@ class HistoryGC(threading.Thread):
         This is sorted by the last closed time. Returns a list of (timestamp,
         file) tuples.
         """
-        _ = self  # this could be a function but is intimate to this class
         # pylint: disable=no-member
         xdd = builtins.__xonsh_env__.get('XONSH_DATA_DIR')
         xdd = expanduser_abs_path(xdd)
@@ -179,7 +177,7 @@ class HistoryFlusher(threading.Thread):
             ljdump(hist, f, sort_keys=True)
 
 
-class CommandField(abc.Sequence):
+class CommandField(cabc.Sequence):
     """A field in the 'cmds' portion of history."""
 
     def __init__(self, field, hist, default=None):
@@ -237,93 +235,72 @@ class CommandField(abc.Sequence):
         return self is self.hist._queue[0]
 
 
-def _find_histfile_var(file_list=None, default=None):
-    if file_list is None:
-        return None
-    hist_file = None
-
-    found_hist = False
+def _find_histfile_var(file_list, default=None):
+    """Return the path of the history file
+    from the value of the envvar HISTFILE.
+    """
     for f in file_list:
         f = expanduser_abs_path(f)
         if not os.path.isfile(f):
             continue
         with open(f, 'r') as rc_file:
             for line in rc_file:
-                if "HISTFILE=" in line:
-                    evar = line.split(' ', 1)[-1]
-                    hist_file = evar.split('=', 1)[-1]
-                    for char in ['"', "'", '\n']:
-                        hist_file = hist_file.replace(char, '')
+                if line.startswith('HISTFILE='):
+                    hist_file = line.split('=', 1)[1].strip('\'"\n')
                     hist_file = expanduser_abs_path(hist_file)
                     if os.path.isfile(hist_file):
-                        found_hist = True
-                        break
-        if found_hist:
-            break
-
-    if hist_file is None:
-        default = expanduser_abs_path(default)
-        if os.path.isfile(default):
-            hist_file = default
-
-    return hist_file
+                        return hist_file
+    else:
+        if default:
+            default = expanduser_abs_path(default)
+            if os.path.isfile(default):
+                return default
 
 
 def _all_xonsh_parser(**kwargs):
     """
     Returns all history as found in XONSH_DATA_DIR.
 
-    return format: (name, start_time, index)
+    return format: (cmd, start_time, index)
     """
     data_dir = builtins.__xonsh_env__.get('XONSH_DATA_DIR')
     data_dir = expanduser_abs_path(data_dir)
 
     files = [os.path.join(data_dir, f) for f in os.listdir(data_dir)
              if f.startswith('xonsh-') and f.endswith('.json')]
-    file_hist = []
+    ind = 0
     for f in files:
         try:
             json_file = LazyJSON(f, reopen=False)
-            file_hist.append(json_file.load()['cmds'])
         except ValueError:
             # Invalid json file
             pass
-    commands = [(c['inp'][:-1] if c['inp'].endswith('\n') else c['inp'],
-                 c['ts'][0])
-                for commands in file_hist for c in commands if c]
-    commands.sort(key=operator.itemgetter(1))
-    return [(c, t, ind) for ind, (c, t) in enumerate(commands)]
+        commands = json_file.load()['cmds']
+        for c in commands:
+            yield (c['inp'].rstrip(), c['ts'][0], ind)
+            ind += 1
 
 
 def _curr_session_parser(hist=None, **kwargs):
     """
     Take in History object and return command list tuple with
-    format: (name, start_time, index)
+    format: (cmd, start_time, index)
     """
     if hist is None:
         hist = builtins.__xonsh_history__
-    if not hist:
-        return None
-    start_times = [start for start, end in hist.tss]
-    names = [name[:-1] if name.endswith('\n') else name
-             for name in hist.inps]
-    commands = enumerate(zip(names, start_times))
-    return [(c, t, ind) for ind, (c, t) in commands]
+    return iter(hist)
 
 
 def _zsh_hist_parser(location=None, **kwargs):
-    default_location = os.path.join('~', '.zsh_history')
-    location_list = [os.path.join('~', '.zshrc'),
-                     os.path.join('~', '.zprofile')]
+    """Yield commands from zsh history file"""
     if location is None:
-        location = _find_histfile_var(location_list, default_location)
-    z_hist_formatted = []
-    if location and os.path.isfile(location):
-        with open(location, 'r', errors='backslashreplace') as z_file:
-            z_txt = z_file.read()
-            z_hist = z_txt.splitlines()
-            if z_hist:
-                for ind, line in enumerate(z_hist):
+        location = _find_histfile_var([os.path.join('~', '.zshrc'),
+                                       os.path.join('~', '.zprofile')],
+                                      os.path.join('~', '.zsh_history'))
+    if location:
+        with open(location, 'r', errors='backslashreplace') as zsh_hist:
+            for ind, line in enumerate(zsh_hist):
+                if line.startswith(':'):
                     try:
                         start_time, command = line.split(';', 1)
                     except ValueError:
@@ -332,29 +309,25 @@ def _zsh_hist_parser(location=None, **kwargs):
                     try:
                         start_time = float(start_time.split(':')[1])
                     except ValueError:
-                        start_time = -1
-                    z_hist_formatted.append((command, start_time, ind))
-                return z_hist_formatted
+                        start_time = 0.0
+                    yield (command.rstrip(), start_time, ind)
+                else:
+                    yield (line.rstrip(), 0.0, ind)
 
     else:
         print("No zsh history file found", file=sys.stderr)
 
 
 def _bash_hist_parser(location=None, **kwargs):
-    default_location = os.path.join('~', '.bash_history')
-    location_list = [os.path.join('~', '.bashrc'),
-                     os.path.join('~', '.bash_profile')]
+    """Yield commands from bash history file"""
     if location is None:
-        location = _find_histfile_var(location_list, default_location)
-    bash_hist_formatted = []
-    if location and os.path.isfile(location):
-        with open(location, 'r', errors='backslashreplace') as bash_file:
-            b_txt = bash_file.read()
-            bash_hist = b_txt.splitlines()
-            if bash_hist:
-                for ind, command in enumerate(bash_hist):
-                    bash_hist_formatted.append((command, 0.0, ind))
-                return bash_hist_formatted
+        location = _find_histfile_var([os.path.join('~', '.bashrc'),
+                                       os.path.join('~', '.bash_profile')],
+                                      os.path.join('~', '.bash_history'))
+    if location:
+        with open(location, 'r', errors='backslashreplace') as bash_hist:
+            for ind, line in enumerate(bash_hist):
+                yield (line.rstrip(), 0.0, ind)
     else:
         print("No bash history file", file=sys.stderr)
 
@@ -366,14 +339,23 @@ def _hist_create_parser():
                                 description='Tools for dealing with history')
     subp = p.add_subparsers(title='action', dest='action')
     # session action
-    show = subp.add_parser('show', aliases=['session'],
+    show = subp.add_parser('show', prefix_chars='-+',
                            help='displays session history, default action')
     show.add_argument('-r', dest='reverse', default=False,
-                      action='store_true',
-                      help='reverses the direction')
+                      action='store_true', help='reverses the direction')
+    show.add_argument('-n', dest='numerate', default=False, action='store_true',
+                      help='numerate each command')
+    show.add_argument('-t', dest='timestamp', default=False,
+                      action='store_true', help='show command timestamps')
+    show.add_argument('-T', dest='end_time', default=None,
+                      help='show only commands before timestamp')
+    show.add_argument('+T', dest='start_time', default=None,
+                      help='show only commands after timestamp')
+    show.add_argument('-f', dest='datetime_format', default=None,
+                      help='the datetime format to be used for filtering and printing')
     show.add_argument('session', nargs='?', choices=_HIST_SESSIONS.keys(), default='session',
                       help='Choose a history session, defaults to current session')
-    show.add_argument('slices', nargs=argparse.REMAINDER, default=[],
+    show.add_argument('slices', nargs='*', default=None,
                       help='display history entries or range of entries')
     # 'id' subcommand
     subp.add_parser('id', help='displays the current session id')
@@ -408,105 +390,131 @@ def _hist_create_parser():
     return p
 
 
-def _hist_show(ns=None, hist=None, start_index=None, end_index=None,
-          start_time=None, end_time=None, location=None):
-    """Show the requested portion of shell history.
-    Accepts multiple history sources (xonsh, bash, zsh)
+def _hist_filter_ts(commands, start_time, end_time):
+    """Yield only the commands between start and end time."""
+    for cmd in commands:
+        if start_time <= cmd[1] < end_time:
+            yield cmd
 
-    May be invoked as an alias with history all/bash/zsh which will
-    provide history as stdout or with __xonsh_history__.show()
-    which will return the history as a list with each item
-    in the tuple form (name, start_time, index).
 
-    If invoked via __xonsh_history__.show() then the ns parameter
-    can be supplied as a str with the follow options::
+def _hist_get(session='session', *, slices=None, datetime_format=None,
+              start_time=None, end_time=None, location=None):
+    """Get the requested portion of shell history.
 
-        session - returns xonsh history from current session
-        xonsh   - returns xonsh history from all sessions
-        all     - alias of xonsh
-        zsh     - returns all zsh history
-        bash    - returns all bash history
+    Parameters
+    ----------
+    session: {'session', 'all', 'xonsh', 'bash', 'zsh'}
+        The history session to get.
+    slices : list of slice-like objects, optional
+        Get only portions of history.
+    start_time, end_time: float, optional
+        Filter commands by timestamp.
+    location: string, optional
+        The history file location (bash or zsh)
+
+    Returns
+    -------
+    generator
+       A filtered list of commands
+    """
+    cmds = _HIST_SESSIONS[session](location=location)
+    if slices:
+        # transform/check all slices
+        slices = [ensure_slice(s) for s in slices]
+        cmds = get_portions(cmds, slices)
+    if start_time or end_time:
+        if start_time is None:
+            start_time = 0.0
+        else:
+            start_time = ensure_timestamp(start_time, datetime_format)
+        if end_time is None:
+            end_time = float('inf')
+        else:
+            end_time = ensure_timestamp(end_time, datetime_format)
+        cmds = _hist_filter_ts(cmds, start_time, end_time)
+    return cmds
+
+
+def _hist_show(ns, *args, **kwargs):
+    """Show the requested portion of shell history.
+    Accepts same parameters with `_hist_get`.
     """
-    # Check if ns is a string, meaning it was invoked from
-    if hist is None:
-        hist = bultins.__xonsh_history__
-    alias = True
-    if isinstance(ns, str) and ns in _HIST_SESSIONS:
-        ns = _hist_create_parser().parse_args([ns])
-        alias = False
-    if not ns:
-        ns = _hist_create_parser().parse_args(['show', 'xonsh'])
-        alias = False
     try:
-        commands = _HIST_SESSIONS[ns.session](hist=hist, location=location)
-    except KeyError:
-        print("{} is not a valid history session".format(ns.action))
-        return None
-    if not commands:
-        return None
-    if start_time:
-        if isinstance(start_time, datetime.datetime):
-            start_time = start_time.timestamp()
-        if isinstance(start_time, float):
-            commands = [c for c in commands if c[1] >= start_time]
-        else:
-            print("Invalid start time, must be float or datetime.")
-    if end_time:
-        if isinstance(end_time, datetime.datetime):
-            end_time = end_time.timestamp()
-        if isinstance(end_time, float):
-            commands = [c for c in commands if c[1] <= end_time]
-        else:
-            print("Invalid end time, must be float or datetime.")
-    idx = None
-    if ns:
-        _commands = []
-        for s in ns.slices:
-            try:
-                s = ensure_slice(s)
-            except (ValueError, TypeError):
-                print('{!r} is not a valid slice format'.format(s), file=sys.stderr)
-                return
-            if s:
-                try:
-                    _commands.extend(commands[s])
-                except IndexError:
-                    err = "Index likely not in range. Only {} commands."
-                    print(err.format(len(commands)), file=sys.stderr)
-                    return
-        else:
-            if _commands:
-                commands = _commands
+        commands = _hist_get(ns.session,
+                             slices=ns.slices,
+                             start_time=ns.start_time,
+                             end_time=ns.end_time,
+                             datetime_format=ns.datetime_format)
+    except ValueError as err:
+        print("history: error: {}".format(err), file=sys.stderr)
+        return
+    if ns.reverse:
+        commands = reversed(list(commands))
+    if not ns.numerate and not ns.timestamp:
+        for c, _, _ in commands:
+            print(c)
+    elif not ns.timestamp:
+        for c, _, i in commands:
+            print('{}: {}'.format(i, c))
+    elif not ns.numerate:
+        for c, ts, _ in commands:
+            dt = datetime.datetime.fromtimestamp(ts).ctime()
+            print('({}) {}'.format(dt, c))
     else:
-        idx = slice(start_index, end_index)
-
-    if (isinstance(idx, slice) and
-            start_time is None and end_time is None):
-        commands = commands[idx]
-
-    if ns and ns.reverse:
-        commands = list(reversed(commands))
-
-    if commands:
-        digits = len(str(max([i for c, t, i in commands])))
-        if alias:
-            for c, t, i in commands:
-                for line_ind, line in enumerate(c.split('\n')):
-                    if line_ind == 0:
-                        print('{:>{width}}: {}'.format(i, line,
-                                                       width=digits + 1))
-                    else:
-                        print(' {:>>{width}} {}'.format('', line,
-                                                        width=digits + 1))
-        else:
-            return commands
+        for c, ts, i in commands:
+            dt = datetime.datetime.fromtimestamp(ts).ctime()
+            print('{}:({}) {}'.format(i, dt, c))
 
 
-#
 # Interface to History
-#
 class History(object):
-    """Xonsh session history."""
+    """Xonsh session history.
+
+    Indexing
+    --------
+    History object acts like a sequence that can be indexed in a special way
+    that adds extra functionality. At the moment only history from the
+    current session can be retrieved. Note that the most recent command
+    is the last item in history.
+
+    The index acts as a filter with two parts, command and argument,
+    separated by comma.  Based on the type of each part different
+    filtering can be achieved,
+
+        for the command part:
+
+            - an int returns the command in that position.
+            - a slice returns a list of commands.
+            - a string returns the most recent command containing the string.
+
+        for the argument part:
+
+            - an int returns the argument of the command in that position.
+            - a slice returns a part of the command based on the argument
+              position.
+
+    The argument part of the filter can be omitted but the command part is
+    required.
+
+    Command arguments are separated by white space.
+
+    If the filtering produces only one result it is
+    returned as a string else a list of strings is returned.
+
+    Attributes
+    ----------
+    rtns : sequence of ints
+        The return of the command (ie, 0 on success)
+    inps : sequence of strings
+        The command as typed by the user, including newlines
+    tss : sequence of two-tuples of floats
+        The timestamps of when the command started and finished, including
+        fractions
+    outs : sequence of strings
+        The output of the command, if xonsh is configured to save it
+
+    In all of these sequences, index 0 is the oldest and -1 (the last item) is the newest.
+    """
 
     def __init__(self, filename=None, sessionid=None, buffersize=100, gc=True,
                  **meta):
@@ -610,18 +618,66 @@ class History(object):
         self.buffer.clear()
         return hf
 
-    def show(self, *args, **kwargs):
-        """
-        Returns shell history as a list
+    def __iter__(self):
+        """Get current session history.
 
-        Valid options:
-            `session` - returns xonsh history from current session
-            `show`    - alias of `session`
-            `xonsh`   - returns xonsh history from all sessions
-            `zsh`     - returns all zsh history
-            `bash`    - returns all bash history
+        Yields
+        ------
+        tuple
+            ``tuple`` of the form (cmd, start_time, index).
         """
-        return _hist_show(*args, **kwargs)
+        start_times = (start for start, end in self.tss)
+        names = (name.rstrip() for name in self.inps)
+        for ind, (c, t) in enumerate(zip(names, start_times)):
+            yield (c, t, ind)
+
+    def __getitem__(self, item):
+        """Retrieve history parts based on filtering rules,
+        see ``History`` docs for more info. Accepts one of
+        int, string, slice or tuple of length two.
+        """
+        if isinstance(item, tuple):
+            cmd_pat, arg_pat = item
+        else:
+            cmd_pat, arg_pat = item, None
+        cmds = (c for c, *_ in self)
+        cmds = self._cmd_filter(cmds, cmd_pat)
+        if arg_pat is not None:
+            cmds = self._args_filter(cmds, arg_pat)
+        cmds = list(cmds)
+        if len(cmds) == 1:
+            return cmds[0]
+        else:
+            return cmds
+
+    @staticmethod
+    def _cmd_filter(cmds, pat):
+        if isinstance(pat, (int, slice)):
+            s = ensure_slice(pat)
+            yield from get_portions(cmds, s)
+        elif is_string(pat):
+            for command in reversed(list(cmds)):
+                if pat in command:
+                    yield command
+        else:
+            raise TypeError('Command filter must be '
+                            'string, int or slice')
+
+    @staticmethod
+    def _args_filter(cmds, pat):
+        args = None
+        if isinstance(pat, (int, slice)):
+            s = ensure_slice(pat)
+            for command in cmds:
+                yield ' '.join(command.split()[s])
+        else:
+            raise TypeError('Argument filter must be '
+                            'int or slice')
+        return args
+
+    def __setitem__(self, *args):
+        raise PermissionError('You cannot change history! '
+                              'you can create new though.')
 
 
 def _hist_info(ns, hist):
@@ -655,35 +711,48 @@ def _HIST_SESSIONS():
             'all': _all_xonsh_parser,
             'zsh': _zsh_hist_parser,
             'bash': _bash_hist_parser}
+
+
 @lazyobject
 def _HIST_MAIN_ACTIONS():
     return {
-    'show': _hist_show,
-    'id': lambda ns, hist: print(hist.sessionid),
-    'file': lambda ns, hist: print(hist.filename),
-    'info': _hist_info,
-    'diff': _dh_main_action,
-    'gc': _hist_gc,
+        'show': _hist_show,
+        'id': lambda ns, hist: print(hist.sessionid),
+        'file': lambda ns, hist: print(hist.filename),
+        'info': _hist_info,
+        'diff': _dh_main_action,
+        'gc': _hist_gc,
     }
 
 
 def _hist_parse_args(args):
-    """Parse arguments using the history argument parser."""
+    """Prepare and parse arguments for the history command.
+
+    Add default action for ``history`` and
+    default session for ``history show``.
+    """
     parser = _hist_create_parser()
     if not args:
-        args = ['show']
-    elif args[0] not in ['-h', '--help'] and args[0] not in _HIST_MAIN_ACTIONS:
-        args.insert(0, 'show')
-    if (args[0] == 'show'
-       and len(args) > 1
-       and args[1] not in ['-h', '--help', '-r']
-       and args[1] not in _HIST_SESSIONS):
-        args.insert(1, 'session')
-    return parser.parse_args(args)
+        args = ['show', 'session']
+    elif args[0] not in _HIST_MAIN_ACTIONS and args[0] not in ('-h', '--help'):
+        args = ['show', 'session'] + args
+    if args[0] == 'show':
+        if not any(a in _HIST_SESSIONS for a in args):
+            args.insert(1, 'session')
+        ns, slices = parser.parse_known_args(args)
+        if slices:
+            if not ns.slices:
+                ns.slices = slices
+            else:
+                ns.slices.extend(slices)
+    else:
+        ns = parser.parse_args(args)
+    return ns
 
 
 def history_main(args=None, stdin=None):
     """This is the history command entry point."""
     hist = builtins.__xonsh_history__
     ns = _hist_parse_args(args)
-    _HIST_MAIN_ACTIONS[ns.action](ns, hist)
+    if ns:
+        _HIST_MAIN_ACTIONS[ns.action](ns, hist)
