@@ -11,11 +11,11 @@ try:
 except ImportError:
     from xonsh.ply.lex import LexToken
 
-from xonsh.lazyasd import LazyObject, lazyobject
+from xonsh.lazyasd import lazyobject
 from xonsh.platform import PYTHON_VERSION_INFO
 from xonsh.tokenize import (OP, IOREDIRECT, STRING, DOLLARNAME, NUMBER,
-    SEARCHPATH, NEWLINE, INDENT, DEDENT, NL, COMMENT, ENCODING,
-    ENDMARKER, NAME, ERRORTOKEN, tokenize, TokenError)
+                            SEARCHPATH, NEWLINE, INDENT, DEDENT, NL, COMMENT, ENCODING,
+                            ENDMARKER, NAME, ERRORTOKEN, tokenize, TokenError)
 
 
 @lazyobject
@@ -145,7 +145,9 @@ def handle_error_token(state, token):
     Function for handling error tokens
     """
     state['last'] = token
-    if not state['pymode'][-1][0]:
+    if token.string == '!':
+        typ = 'BANG'
+    elif not state['pymode'][-1][0]:
         typ = 'NAME'
     else:
         typ = 'ERRORTOKEN'
@@ -169,6 +171,7 @@ def _make_matcher_handler(tok, typ, pymode, ender, handlers):
     matcher = (')' if tok.endswith('(') else
                '}' if tok.endswith('{') else
                ']' if tok.endswith('[') else None)
+
     def _inner_handler(state, token):
         state['pymode'].append((pymode, tok, matcher, token.start))
         state['last'] = token
@@ -339,6 +342,7 @@ class Lexer(object):
         if self._tokens is None:
             t = tuple(token_map.values()) + (
                 'NAME',                  # name tokens
+                'BANG',                  # ! tokens
                 'WS',                    # whitespace in subprocess mode
                 'LPAREN', 'RPAREN',      # ( )
                 'LBRACKET', 'RBRACKET',  # [ ]
