@@ -10,7 +10,7 @@ import functools
 import subprocess
 import importlib.util
 
-from xonsh.lazyasd import LazyObject, LazyBool, lazyobject, lazybool
+from xonsh.lazyasd import LazyBool, lazyobject, lazybool
 
 
 @lazyobject
@@ -46,6 +46,12 @@ ON_POSIX = LazyBool(lambda: (os.name == 'posix'), globals(), 'ON_POSIX')
 ON_FREEBSD = LazyBool(lambda: (sys.platform.startswith('freebsd')),
                       globals(), 'ON_FREEBSD')
 """``True`` if on a FreeBSD operating system, else ``False``."""
+ON_NETBSD = LazyBool(lambda: (sys.platform.startswith('netbsd')),
+                     globals(), 'ON_NETBSD')
+"""``True`` if on a NetBSD operating system, else ``False``."""
+ON_BSD = LazyBool(lambda: ON_FREEBSD or ON_NETBSD,
+                  globals(), 'ON_BSD')
+"""``True`` if on a BSD operating system, else ``False``."""
 
 
 #
@@ -227,8 +233,7 @@ def git_for_windows_path():
 
 @functools.lru_cache(1)
 def windows_bash_command():
-    """Determines teh command for Bash on windows."""
-    import winreg
+    """Determines the command for Bash on windows."""
     # Check that bash is on path otherwise try the default directory
     # used by Git for windows
     wbc = 'bash'
@@ -248,6 +253,17 @@ def windows_bash_command():
 # Environment variables defaults
 #
 
+
+@functools.lru_cache(1)
+def bash_command():
+    """Determines the command for Bash on the current plaform."""
+    if ON_WINDOWS:
+        bc = windows_bash_command()
+    else:
+        bc = 'bash'
+    return bc
+
+
 @lazyobject
 def BASH_COMPLETIONS_DEFAULT():
     """A possibly empty tuple with default paths to Bash completions known for
@@ -255,22 +271,15 @@ def BASH_COMPLETIONS_DEFAULT():
     """
     if ON_LINUX or ON_CYGWIN:
         if linux_distro() == 'arch':
-            bcd = (
-                '/usr/share/bash-completion/bash_completion',
-                '/usr/share/bash-completion/completions')
+            bcd = ('/usr/share/bash-completion/bash_completion', )
         else:
-            bcd = ('/usr/share/bash-completion',
-                   '/usr/share/bash-completion/completions')
+            bcd = ('/usr/share/bash-completion', )
     elif ON_DARWIN:
-        bcd = ('/usr/local/etc/bash_completion',
-               '/opt/local/etc/profile.d/bash_completion.sh')
+        bcd = ('/usr/local/share/bash-completion/bash_completion',  # v2.x
+               '/usr/local/etc/bash_completion')  # v1.x
     elif ON_WINDOWS and git_for_windows_path():
         bcd = (os.path.join(git_for_windows_path(),
-                    'usr\\share\\bash-completion'),
-               os.path.join(git_for_windows_path(),
-                    'usr\\share\\bash-completion\\completions'),
-               os.path.join(git_for_windows_path(),
-                     'mingw64\\share\\git\\completion\\git-completion.bash'))
+                            'usr\\share\\bash-completion'), )
     else:
         bcd = ()
     return bcd
@@ -292,7 +301,7 @@ def PATH_DEFAULT():
     elif ON_WINDOWS:
         import winreg
         key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE,
-                r'SYSTEM\CurrentControlSet\Control\Session Manager\Environment')
+                             r'SYSTEM\CurrentControlSet\Control\Session Manager\Environment')
         pd = tuple(winreg.QueryValueEx(key, 'Path')[0].split(os.pathsep))
     else:
         pd = ()
