@@ -19,21 +19,21 @@ Implementations:
 """
 import builtins
 import collections
-import collections.abc as abc
+import collections.abc as cabc
+import contextlib
 import ctypes
+import datetime
 import functools
 import glob
+import itertools
 import os
 import pathlib
 import re
-import string
 import subprocess
 import sys
 import threading
 import traceback
 import warnings
-import contextlib
-import subprocess
 
 # adding further imports from xonsh modules is discouraged to avoid circular
 # dependencies
@@ -42,7 +42,6 @@ from xonsh.platform import (has_prompt_toolkit, scandir, DEFAULT_ENCODING,
                             ON_LINUX, ON_WINDOWS, PYTHON_VERSION_INFO)
 
 
-
 @functools.lru_cache(1)
 def is_superuser():
     if ON_WINDOWS:
@@ -245,6 +244,9 @@ def find_next_break(line, mincol=0, lexer=None):
         elif tok.type == 'ERRORTOKEN' and ')' in tok.value:
             maxcol = tok.lexpos + mincol + 1
             break
+        elif tok.type == 'BANG':
+            maxcol = mincol + len(line) + 1
+            break
     return maxcol
 
 
@@ -261,11 +263,17 @@ def subproc_toks(line, mincol=-1, maxcol=None, lexer=None, returnline=False):
     lexer.input(line)
     toks = []
     lparens = []
+    saw_macro = False
     end_offset = 0
     for tok in lexer:
         pos = tok.lexpos
         if tok.type not in END_TOK_TYPES and pos >= maxcol:
             break
+        if tok.type == 'BANG':
+            saw_macro = True
+        if saw_macro and tok.type not in ('NEWLINE', 'DEDENT'):
+            toks.append(tok)
+            continue
         if tok.type in LPARENS:
             lparens.append(tok.type)
         if len(toks) == 0 and tok.type in BEG_TOK_SKIPS:
@@ -315,6 +323,8 @@ def subproc_toks(line, mincol=-1, maxcol=None, lexer=None, returnline=False):
             end_offset = len(el)
     if len(toks) == 0:
         return  # handle comment lines
+    elif saw_macro:
+        end_offset = len(toks[-1].value.rstrip()) + 1
     beg, end = toks[0].lexpos, (toks[-1].lexpos + end_offset)
     end = len(line[:end].rstrip())
     rtn = '![' + line[beg:end] + ']'
@@ -542,7 +552,7 @@ def command_not_found(cmd):
     c = '/usr/lib/command-not-found {0}; exit 0'
     s = subprocess.check_output(c.format(cmd), universal_newlines=True,
                                 stderr=subprocess.STDOUT, shell=True)
-    s = '\n'.join(s.splitlines()[:-1]).strip()
+    s = '\n'.join(s.rstrip().splitlines()).strip()
     return s
 
 
@@ -596,7 +606,7 @@ def print_exception(msg=None):
     if env is None:
         env = os.environ
         manually_set_trace = 'XONSH_SHOW_TRACEBACK' in env
-        manually_set_logfile ='XONSH_TRACEBACK_LOGFILE' in env
+        manually_set_logfile = 'XONSH_TRACEBACK_LOGFILE' in env
     else:
         manually_set_trace = env.is_manually_set('XONSH_SHOW_TRACEBACK')
         manually_set_logfile = env.is_manually_set('XONSH_TRACEBACK_LOGFILE')
@@ -923,18 +933,23 @@ def bool_or_int_to_str(x):
     return bool_to_str(x) if is_bool(x) else str(x)
 
 
-
 @lazyobject
 def SLICE_REG():
     return re.compile(r'(?P<start>(?:-\d)?\d*):(?P<end>(?:-\d)?\d*):?(?P<step>(?:-\d)?\d*)')
 
+
 def ensure_slice(x):
-    """Convert a string or int to a slice."""
-    if x is None:
+    """Try to convert an object into a slice, complain on failure"""
+    if not x and x != 0:
         return slice(None)
+    elif is_slice(x):
+        return x
     try:
         x = int(x)
-        s = slice(x, x+1)
+        if x != -1:
+            s = slice(x, x + 1)
+        else:
+            s = slice(-1, None, None)
     except ValueError:
         x = x.strip('[]()')
         m = SLICE_REG.fullmatch(x)
@@ -944,10 +959,35 @@ def ensure_slice(x):
         else:
             raise ValueError('cannot convert {!r} to slice'.format(x))
     except TypeError:
-        raise TypeError('ensure_slice() argument must be a string or a number not {}'.format(type(x)))
+        try:
+            s = slice(*(int(i) for i in x))
+        except (TypeError, ValueError):
+            raise ValueError('cannot convert {!r} to slice'.format(x))
     return s
 
 
+def get_portions(it, slices):
+    """Yield from portions of an iterable.
+
+    Parameters
+    ----------
+    it: iterable
+    slices: a slice or a list of slice objects
+    """
+    if is_slice(slices):
+        slices = [slices]
+    if len(slices) == 1:
+        s = slices[0]
+        try:
+            yield from itertools.islice(it, s.start, s.stop, s.step)
+            return
+        except ValueError:  # islice failed
+            pass
+    it = list(it)
+    for s in slices:
+        yield from it[s]
+
+
 def is_slice_as_str(x):
     """
     Test if string x is a slice. If not a string return False.
@@ -974,7 +1014,7 @@ def is_int_as_str(x):
 
 def is_string_set(x):
     """Tests if something is a set of strings"""
-    return (isinstance(x, abc.Set) and
+    return (isinstance(x, cabc.Set) and
             all(isinstance(a, str) for a in x))
 
 
@@ -1010,7 +1050,7 @@ def set_to_pathsep(x, sort=False):
 
 def is_string_seq(x):
     """Tests if something is a sequence of strings"""
-    return (isinstance(x, abc.Sequence) and
+    return (isinstance(x, cabc.Sequence) and
             all(isinstance(a, str) for a in x))
 
 
@@ -1018,7 +1058,7 @@ def is_nonstring_seq_of_strings(x):
     """Tests if something is a sequence of strings, where the top-level
     sequence is not a string itself.
     """
-    return (isinstance(x, abc.Sequence) and not isinstance(x, str) and
+    return (isinstance(x, cabc.Sequence) and not isinstance(x, str) and
             all(isinstance(a, str) for a in x))
 
 
@@ -1052,7 +1092,7 @@ def seq_to_upper_pathsep(x):
 
 def is_bool_seq(x):
     """Tests if an object is a sequence of bools."""
-    return isinstance(x, abc.Sequence) and all(isinstance(y, bool) for y in x)
+    return isinstance(x, cabc.Sequence) and all(isinstance(y, bool) for y in x)
 
 
 def csv_to_bool_seq(x):
@@ -1171,7 +1211,7 @@ HISTORY_UNITS = LazyObject(lambda: {
 
 def is_history_tuple(x):
     """Tests if something is a proper history value, units tuple."""
-    if (isinstance(x, abc.Sequence) and
+    if (isinstance(x, cabc.Sequence) and
             len(x) == 2 and
             isinstance(x[0], (int, float)) and
             x[1].lower() in CANON_HISTORY_UNITS):
@@ -1218,7 +1258,7 @@ RE_HISTORY_TUPLE = LazyObject(
 
 def to_history_tuple(x):
     """Converts to a canonincal history tuple."""
-    if not isinstance(x, (abc.Sequence, float, int)):
+    if not isinstance(x, (cabc.Sequence, float, int)):
         raise ValueError('history size must be given as a sequence or number')
     if isinstance(x, str):
         m = RE_HISTORY_TUPLE.match(x.strip().lower())
@@ -1269,7 +1309,10 @@ def _get_color_indexes(style_map):
     for token in style_map:
         attr = pt_style.token_to_attrs[token]
         if attr.color is not None:
-            index = table.lookup_color(attr.color, attr.bgcolor)
+            try:
+                index = table.lookup_color(attr.color, attr.bgcolor)
+            except AttributeError:
+                index = table.lookup_fg_color(attr.color)
             try:
                 rgb = (int(attr.color[0:2], 16),
                        int(attr.color[2:4], 16),
@@ -1449,6 +1492,7 @@ if ON_WINDOWS:
     def WINDOWS_ENVVAR_REGEX():
         return re.compile(r"%(?P<envvar>\w+)%")
 
+
 def expandvars(path):
     """Expand shell variables of the forms $var, ${var} and %var%.
     Unknown variables are left unchanged."""
@@ -1582,3 +1626,19 @@ def _iglobpath(s, ignore_case=False, sort_result=None):
 def iglobpath(s, ignore_case=False, sort_result=None):
     """Simple wrapper around iglob that also expands home and env vars."""
     return _iglobpath(s, ignore_case=ignore_case, sort_result=sort_result)[0]
+
+
+def ensure_timestamp(t, datetime_format=None):
+    if isinstance(t, (int, float)):
+        return t
+    try:
+        return float(t)
+    except (ValueError, TypeError):
+        pass
+    if datetime_format is None:
+        datetime_format = builtins.__xonsh_env__['XONSH_DATETIME_FORMAT']
+    if isinstance(t, datetime.datetime):
+        t = t.timestamp()
+    else:
+        t = datetime.datetime.strptime(t, datetime_format).timestamp()
+    return t
