@@ -3,13 +3,13 @@
 import os
 import ast
 import json
+import pprint
 import builtins
 import textwrap
-from pprint import pformat
-from collections.abc import Mapping, Sequence
 
 from xonsh.tools import to_bool, to_bool_or_break, backup_file, print_color
 
+
 #
 # Nodes themselves
 #
@@ -168,6 +168,7 @@ class YesNo(Question):
         super().__init__(question, responses, converter=to_bool,
                          path=path)
 
+
 class TrueFalse(Input):
     """Input node the returns a True or False value."""
 
@@ -190,7 +191,7 @@ class StoreNonEmpty(Input):
     """
 
     def __init__(self, prompt='>>> ', converter=None, show_conversion=False,
-                 confirm=False, retry=False, path=None):
+                 confirm=False, retry=False, path=None, store_raw=False):
         def nonempty_converter(x):
             """Converts non-empty values and converts empty inputs to
             Unstorable.
@@ -199,6 +200,8 @@ class StoreNonEmpty(Input):
                 x = Unstorable
             elif converter is None:
                 pass
+            elif store_raw:
+                converter(x)  # make sure str is valid, even if storing raw
             else:
                 x = converter(x)
             return x
@@ -278,7 +281,10 @@ def create_truefalse_cond(prompt='yes or no [default: no]? ', path=None):
 # Tools for trees of nodes.
 #
 
-_lowername = lambda cls: cls.__name__.lower()
+
+def _lowername(cls):
+    return cls.__name__.lower()
+
 
 class Visitor(object):
     """Super-class for all classes that should walk over a tree of nodes.
@@ -324,7 +330,7 @@ class PrettyFormatter(Visitor):
         t = []
         for aname in node.attrs:
             a = getattr(node, aname)
-            t.append(self.visit(a) if isinstance(a, Node) else pformat(a))
+            t.append(self.visit(a) if isinstance(a, Node) else pprint.pformat(a))
         t = ['{0}={1}'.format(n, x) for n, x in zip(node.attrs, t)]
         s += textwrap.indent(',\n'.join(t), self.indent)
         self.level -= 1
@@ -511,6 +517,7 @@ YN = "{GREEN}yes{NO_COLOR} or {RED}no{NO_COLOR} [default: no]? "
 YNB = ('{GREEN}yes{NO_COLOR}, {RED}no{NO_COLOR}, or '
        '{YELLOW}break{NO_COLOR} [default: no]? ')
 
+
 class PromptVisitor(StateVisitor):
     """Visits the nodes in the tree via the a command-line prompt."""
 
@@ -568,14 +575,14 @@ class PromptVisitor(StateVisitor):
                     else:
                         raise
                 if node.show_conversion and x is not Unstorable \
-                                        and str(x) != raw:
+                        and str(x) != raw:
                     msg = '{{BOLD_PURPLE}}Converted{{NO_COLOR}} input {0!r} to {1!r}.'
                     print_color(msg.format(raw, x))
             else:
                 x = raw
             if node.confirm:
                 msg = 'Would you like to keep the input: {0}'
-                print(msg.format(pformat(x)))
+                print(msg.format(pprint.pformat(x)))
                 confirmer = TrueFalseBreak(prompt=YNB)
                 status = self.visit(confirmer)
                 if isinstance(status, str) and status == 'break':
